/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.cat.glossary.engine.regex.ui;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;
import org.osgi.framework.BundleContext;

import benten.cat.glossary.core.BentenGlossaryDriver;
import benten.cat.glossary.core.BentenGlossaryDriverManager;
import benten.cat.glossary.core.BentenGlossaryEngine;
import benten.cat.glossary.data.AbstractBentenGlossaryDataTsvPlugin;
import benten.cat.glossary.engine.AbstractBentenGlossaryEnginePlugin;
import benten.cat.glossary.engine.regex.RegexGlossaryDriver;

/**
 * 正規表現用語集ドライバー
 *
 * このクラスは、TM エンジンの切り替えテストのためのテストクラスです。
 * 実際の Benten 基本セットには含まれません。
 */
public class RegexGlossaryPlugin extends AbstractBentenGlossaryEnginePlugin {
	private static final String DEFAULT_GLOSSARY = "Default Glossary";

	private final Map<String, AbstractBentenGlossaryDataTsvPlugin> glossaryMap = new HashMap<String, AbstractBentenGlossaryDataTsvPlugin>();

	@Override
	public void start(final BundleContext context) throws Exception {
		super.start(context);
		plugin = this;
	}

	@Override
	public void stop(final BundleContext context) throws Exception {
		plugin = null;
		super.stop(context);
	}

	@Override
	public BentenGlossaryDriver getDriver() {
		try {
			// 自らのドライバーを自己登録します。
			Class.forName("benten.cat.glossary.engine.regex.RegexGlossaryDriver");
		} catch (final ClassNotFoundException e) {
			getDefault().log("Regex グロッサリー・エンジンの読み込みに失敗しました。");
			getDefault().log(e);
		}

		return BentenGlossaryDriverManager.getDriver(RegexGlossaryDriver.DRIVER_NAME);
	}

	@Override
	public BentenGlossaryEngine getEngine() {
		if (fEngine == null) {
			fEngine = getDriver().getEngineInstance();
		}

		return fEngine;
	}

	@SuppressWarnings("static-access")
	@Override
	public void load() {
		getDefault().log("Regex 用語集エンジン・プラグイン: 用語集をリフレッシュ・ロードします。");

		getEngine().unload();

		final IExtensionRegistry registry = Platform.getExtensionRegistry();
		// OmegaT の辞書データを活用しています。
		final IExtensionPoint point = registry.getExtensionPoint("benten.cat.glossary.data.tsv");
		if (point == null)
			return;

		final IExtension[] extensions = point.getExtensions();
		for (int i = 0; i < extensions.length; i++) {
			final IConfigurationElement[] cfgElems = extensions[i].getConfigurationElements();
			for (int j = 0; j < cfgElems.length; j++) {
				final IConfigurationElement cfgElem = cfgElems[j];
				try {
					final String name = cfgElem.getAttribute("name");
					final Object obj = cfgElem.createExecutableExtension("class");
					if (obj instanceof AbstractBentenGlossaryDataTsvPlugin) {
						glossaryMap.put(name, (AbstractBentenGlossaryDataTsvPlugin) obj);
					}
				} catch (final CoreException e) {
					RegexGlossaryPlugin.getDefault().log(e);
				}
			}
		}

		final AbstractBentenGlossaryDataTsvPlugin glossaryDataPlugin = glossaryMap.get(DEFAULT_GLOSSARY);
		if (glossaryDataPlugin == null)
			return;

		try {
			// FIXME ファイルの読み込み方
			final URL entry = glossaryDataPlugin.getDefault().getBundle().getEntry("/glossary");
			final String pluginDirectory = FileLocator.resolve(entry).getPath();
			final File file = new File(pluginDirectory);

			getEngine().loadTsv(file);
		} catch (final IOException e) {
			RegexGlossaryPlugin.getDefault().log(e);
		}
	}
}
